import _ from 'lodash'
import {defineStore} from 'pinia'
import copy from '@/store/copy'
import lock from '@/store/lock'
import layer from '@/store/layer'
import system from "@/utils/system"
import compose from '@/store/compose'
import snapshot from '@/store/snapshot'
import Constant from "@/utils/constant";
import EditorUtils from '@/utils/EditorUtils'
import {ruleStore} from "@/store/ruler-things";
import {tableBands, tableColTemplate} from "@/components/config/globalConfig";

export const globalStore = defineStore('global', {
    state: () => {
        return {
            ...compose.state,
            ...snapshot.state,
            ...copy.state,
            // 编辑器模式 edit preview
            editMode: 'edit',
            pageConfig: {
                // 页面方向 h 水平方向 v 垂直方向
                pageDirection: 'Vertical',
                // 页面长度：px
                pageWidth: 595,
                // 页面高度：px
                pageHeight: 842,
                pageSize: 'A4',
                // 模板标题
                title: '新建模板',
                // 模板编码
                code: 'TEMPLATE',
                // 默认缩放比例：100%
                // 页面边距 px
                topMargin: 20,
                rightMargin: 20,
                bottomMargin: 20,
                leftMargin: 20,
                style: {
                    // 默认缩放比例：100%
                    scale: 1,
                    // 页面背景
                    background: '#ffffff',
                    // 默认字体颜色
                    color: '#000000',
                    // 默认字号
                    fontSize: 12,
                    // 默认字体
                    fontName: '微软雅黑',
                    // 默认行高
                    lineHeight: 1,
                }
            },
            // 是否在编辑器中，用于判断复制、粘贴组件时是否生效，如果在编辑器外，则无视这些操作
            isInEditor: false,
            componentData: [],
            elementBands: new Map(),
            elementBandCodes: [],
            dataSources: [],
            dataset: {},
            curComponent: null,
            curTableCell: null,
            curComponentIndex: null,
            // 点击画布时是否点中组件，主要用于取消选中组件用。
            // 如果没点中组件，并且在画布空白处弹起鼠标，则取消当前组件的选中状态
            isClickComponent: false,
            curTableSettingId: null,
            selectedTableCell: {
                bandRowName: "",
                rowNo: -1,
                colNo: -1,
                element: {}
            },
            editorMenuInfo: {
                visible: false,
                style: {
                    top: 0,
                    left: 0
                }
            }
        }
    },
    actions: {
        ...layer.actions,
        ...compose.actions,
        ...snapshot.actions,
        ...copy.actions,
        ...lock.actions,
        setClickComponentStatus(status) {
            this.isClickComponent = status
        },

        setEditMode(mode) {
            this.editMode = mode
        },

        setInEditorStatus(status) {
            this.isInEditor = status
        },

        setCanvasStyle(style) {
            this.canvasStyleData = style
        },

        setCurComponent({component, index}) {
            this.curComponent = component
            this.curComponentIndex = index
            this.curTableCell = null
        },
        setSelectedTableCell({bandRowName, rowNo, colNo, element}) {
            this.selectedTableCell = {bandRowName, rowNo, colNo, element}
        },
        getSelectedTableCell() {
            return this.selectedTableCell;
        },
        setPageSize({pageWidth, pageHeight}) {
            this.pageConfig.pageWidth = pageWidth
            this.pageConfig.pageHeight = pageHeight
            ruleStore().setRect({pageWidth, pageHeight})
            ruleStore().setReDrawRuler()
        },
        togglePageDirection(pageDirection) {
            const pageConfig = this.pageConfig
            pageConfig.pageDirection = pageDirection
            ruleStore().setRect({
                pageWidth: pageDirection === 'Vertical' ? pageConfig.pageWidth : pageConfig.pageHeight,
                pageHeight: pageDirection === 'Vertical' ? pageConfig.pageHeight : pageConfig.pageWidth
            })
            ruleStore().setReDrawRuler()
        },

        setPageConfig(pageConfig) {
            this.pageConfig = pageConfig
            ruleStore().setRect({
                pageWidth: pageConfig.pageDirection === 'p' ? pageConfig.pageWidth : pageConfig.pageHeight,
                pageHeight: pageConfig.pageDirection === 'p' ? pageConfig.pageHeight : pageConfig.pageWidth
            })
            ruleStore().setReDrawRuler()
        },

        updateDataValue({data, key, value}) {
            data[key] = value
        },
        setExpression({bandCode, expression, id}) {
            const elementBand = this.elementBands.get(bandCode)
            if (elementBand) {
                let newElementValue = null;
                let newElementIndex = null;
                const elements = elementBand.elements
                for (let i = 0; i < elements?.length; i++) {
                    if (elements[i].id === id) {
                        newElementIndex = i
                        newElementValue = elements[i]
                        newElementValue.expression = expression
                    }
                    if (newElementIndex !== null) {
                        elements[newElementIndex] = newElementValue
                        this.recordSnapshot()
                    }
                }
            }
            this.curComponent.expression = expression;
        },
        setPropValue({propValue, id}) {
            const componentData = this.componentData
            if (this.componentData.length) {
                let newComponentValue = null
                let newComponentIndex = null
                for (let i = 0; i < componentData.length; i++) {
                    if (componentData[i].id === id) {
                        newComponentIndex = i
                        newComponentValue = componentData[i]
                        newComponentValue.propValue = propValue
                    }
                }
                if (newComponentIndex !== null) {
                    this.componentData[newComponentIndex] = newComponentValue
                    this.recordSnapshot()
                }
            }
        },

        setBindValue({bindValue, id}) {
            const componentData = this.componentData
            if (componentData.length) {
                let newComponentValue = null
                let newComponentIndex = null
                for (let i = 0; i < componentData.length; i++) {
                    if (componentData[i].id === id) {
                        newComponentIndex = i
                        newComponentValue = componentData[i]
                        newComponentValue.bindValue = bindValue
                    }
                }
                if (newComponentIndex !== null) {
                    this.componentData[newComponentIndex] = newComponentValue
                    this.recordSnapshot()
                }
            }
        },

        addDataSource(dataSource) {
            this.dataSources.push(dataSource);
        },
        setShapeStyle({top, left, width, height, rotate}) {
            if (top) {
                this.curComponent.style.top = Math.round(top)
            }
            if (left) {
                this.curComponent.style.left = Math.round(left)
            }
            if (width) {
                this.curComponent.style.width = Math.round(width)
            }
            if (height) {
                this.curComponent.style.height = Math.round(height)
            }
            if (rotate) {
                this.curComponent.style.rotate = Math.round(rotate)
            }
        },
        setShapeSingleStyle({key, value}) {
            this.curComponent.style[key] = value
        },
        setComponentData(componentData = []) {
            this.componentData = componentData
        },
        getElementBandByCode(bandCode) {
            return this.elementBands.get(bandCode);
        },
        addElementBand(elementBand) {
            this.elementBands.set(elementBand.code, {
                band: {height: elementBand.height, splitType: 'Stretch'},
                elements: []
            })
        },
        setElementBandCode({elementBandIdx, elementBandCode}) {
            const elementBandCodes = this.elementBandCodes;
            elementBandCodes.splice(elementBandIdx, 1, elementBandCode);
            this.elementBandCodes = elementBandCodes;
        },
        deleteElementBandCode({elementBandCode}) {
            const elementBandCodes = this.elementBandCodes;
            for (let elementBandIdx in elementBandCodes) {
                const bandCode = elementBandCodes[elementBandIdx];
                if (bandCode === elementBandCode) {
                    this.elementBandCodes.splice(Number(elementBandIdx), 1, Constant.NONE)
                }
            }
        },
        getElementBandCodes() {
            const elementBandCodes = this.elementBandCodes;
            if (!elementBandCodes) {
                return [];
            }
            const distElementBandCodes = [];
            for (let elementBandCode of elementBandCodes) {
                if (elementBandCode !== Constant.NONE) {
                    distElementBandCodes.push(elementBandCode)
                }
            }
            return distElementBandCodes;
        },
        changeBandHeight({code, height}) {
            const originElementBand = this.elementBands.get(code)
            if (originElementBand !== undefined) {
                originElementBand.band.height = height;
            }
        },
        deleteElementBand(elementBandCode) {
            if (elementBandCode !== undefined) {
                this.elementBands.delete(elementBandCode)
            }
        },
        addElement({bandCode, element, index}) {
            const elementBand = this.elementBands.get(bandCode);
            if (index !== undefined) {
                elementBand.elements.splice(index, 0, element)
            } else {
                elementBand.elements.push(element)
            }
        },
        deleteElement({element, elementIdx}) {
            const elementBand = this.elementBands.get(element.bandCode);
            if (elementIdx === undefined) {
                elementIdx = this.curComponentIndex
            }
            if (elementIdx === this.curComponentIndex) {
                this.curComponentIndex = null
                this.curComponent = null
            }
            if (/\d/.test(elementIdx)) {
                elementBand.elements.splice(elementIdx, 1)
            }
        },
        addTableCellElement({bandCode, elementId, bandRowName, rowNo, colNo, element}) {
            const elementBand = this.elementBands.get(bandCode);
            // 数组角标从0开始
            rowNo = --rowNo;
            if (elementId !== undefined) {
                const tableElement = elementBand.elements.find(ele => ele.id === elementId);
                if (!tableElement) {
                    system.alert(`未获取到id=${elementId}的元素`)
                    return;
                }
                const col = tableElement.config[bandRowName].rows[rowNo].cols[colNo];
                if (col) {
                    col.element = element
                } else {
                    tableElement.config[bandRowName].rows[rowNo].cols[colNo] = {
                        colspan: 1,
                        rowspan: 1,
                        element: element
                    }
                }
            }
        },
        deleteTableCellElement({bandRowName, rowNo, colNo}) {

        },
        addTableCol({bandCode, elementIdx, colNo}) {
            if (_.isEmpty(bandCode) || _.isNull(elementIdx)) {
                console.error("bandCode or elementIdx must be not null");
                return;
            }
            const element = this.elementBands.get(bandCode).elements[elementIdx];
            if (!element) {
                console.error("elementBand not found");
                return;
            }
            for (let tableBand of tableBands) {
                const tableBandCode = tableBand.code;
                if (!Reflect.has(element.config, tableBandCode)) {
                    continue
                }
                const rows = element.config[tableBandCode].rows
                for (let row of rows) {
                    row.cols[colNo] = system.deepCopy(tableColTemplate)
                }
            }
        },
        removeTableCol({bandCode, elementIdx, colNo}) {
            if (_.isEmpty(bandCode) || _.isNull(elementIdx)) {
                console.error("bandCode or elementIdx must be not null");
                return;
            }
            const element = this.elementBands.get(bandCode).elements[elementIdx];
            if (!element) {
                console.error("elementBand not found");
                return;
            }
            for (let tableBand of tableBands) {
                const tableBandCode = tableBand.code;
                if (!Reflect.has(element.config, tableBandCode)) {
                    continue
                }
                const rows = element.config[tableBandCode].rows
                for (let row of rows) {
                    Reflect.deleteProperty(row.cols, colNo)
                }
            }
        },
        showEditorMenu({top, left}) {
            const editorMenuPos = EditorUtils.getEditorMenuPos(top, left);
            this.editorMenuInfo = {
                visible: true,
                style: {
                    top: editorMenuPos.top,
                    left: editorMenuPos.left
                }
            };
        },
        hideEditorMenu() {
            this.editorMenuInfo = {
                visible: false,
                style: {
                    top: 0,
                    left: 0
                }
            };
        }
    },
    getters: {
        getPageConfig() {
            return this.pageConfig
        },
    }
})
