import {h} from "vue";
import {nanoid} from "nanoid";
import * as FaIcon from '@vicons/fa';
import {ElIcon} from "element-plus";
import {ElLoading} from 'element-plus'
import {ElMessage} from 'element-plus'
import Constant from '@/utils/constant'
import Toast from "@/components/yvan-ui/yvan-print-toast"
import Dialog from "@/components/yvan-ui/yvan-print-dialog"

class System {

    private loadingStack = new Map();

    public getUuid(length = 8) {
        return nanoid(length)
    }

    public toast(message: string = '', type: string = 'warning', duration: number = 3000): void {
        return Toast({
            type: type,
            message: message,
            duration: duration
        })
    }

    public showDialog(componentName: any, props: DialogProps): Promise<any> {
        return Dialog({componentName, ...props})
    }

    // public renderIcon(iconName: string): any {
    //     if (!iconName) {
    //         return
    //     }
    //     return () => h(ElIcon, null, {default: () => h(FaIcon[iconName])});
    // }

    public loading(): string {
        const loadingId = nanoid();
        this.loadingStack.set(loadingId, ElLoading.service({
            lock: true,
            text: '加载中...',
            background: 'rgba(0, 0, 0, 0.7)',
        }))
        return loadingId;
    }

    public unloading(loadingId: string): void {
        if (!loadingId) {
            return;
        }
        const loading = this.loadingStack.get(loadingId)
        if (loading !== undefined) {
            loading.close();
        }
    }

    public alert(message: string = '', type: "" | "success" | "warning" | "error" | "info" = 'info', duration: number = 3000): void {
        console.info(message)
        ElMessage({
            grouping: true,
            message: message,
            type: type,
            duration: duration
        })
    }

    public deepCopy(obj, cache = []) {
        if (obj === null || typeof obj !== 'object') {
            return obj
        }
        const objType = Object.prototype.toString.call(obj).slice(8, -1)
        // 考虑 正则对象的copy
        if (objType === 'RegExp') {
            return new RegExp(obj)
        }
        // 考虑 Date 实例 copy
        if (objType === 'Date') {
            return new Date(obj)
        }
        // 考虑 Error 实例 copy
        if (objType === 'Error') {
            return new Error(obj)
        }
        const hit = cache.filter((c) => c.original === obj)[0]
        if (hit) {
            return hit.copy
        }
        const copy = Array.isArray(obj) ? [] : {}
        cache.push({original: obj, copy})
        Object.keys(obj).forEach((key) => {
            copy[key] = this.deepCopy(obj[key], cache)
        })
        return copy
    }

    /**
     * 打开对话框
     * @param fileUrl       转向网页的地址;
     */
    public openWindow(fileUrl) {
        this.doOpenWindow(fileUrl, '模板预览', Constant.defaultWidth, Constant.defaultHeight);
    }

    /**
     * 打开对话框
     * @param url       转向网页的地址;
     * @param title     网页名称(选填);
     * @param iWidth    弹出窗口的宽度;
     * @param iHeight   弹出窗口的高度;
     */
    public doOpenWindow(url, title, iWidth, iHeight): WindowProxy | null {
        // window.screen.height获得屏幕的高，window.screen.width获得屏幕的宽
        // 获得窗口的垂直位置;
        const iTop = (window.screen.height - 30 - iHeight) / 2;
        // 获得窗口的水平位置;
        const iLeft = (window.screen.width - 10 - iWidth) / 2;
        return window.open(url, title, 'height=' + iHeight + ',innerHeight=' + iHeight + ',width=' + iWidth + ',innerWidth=' + iWidth + ',top=' + iTop + ',left=' + iLeft + ',toolbar=no,menubar=no,scrollbars=auto,resizeable=no,location=no,status=no');
    }
}

export default new System()

export interface DialogProps {
    title: String,
    width: Number,
}
